@@ -25,6 +25,15 @@ from flatland.text.text_block import TextBlock
 
 BranchLeaves = namedtuple('BranchLeaves', 'leaf_stems local_graft next_graft floating_leaf_stem')
 
+def make_node_ref(node_ref) -> str:
+    """
+    Composes a fully qualified placement name based on a node reference in a layout file
+
+    :param node_ref:  A layout reference to a node placement
+    :return: The model elment name if it is only placed once, otherwise name suffixed by an underscore and a numbered placement
+    """
+    return node_ref[0] if len(node_ref) < 2 else f"{node_ref[0]}_{node_ref[1]}"
+
 class XumlClassDiagram:
 
     def __init__(self, xuml_model_path: Path, flatland_layout_path: Path, diagram_file_path: Path,
@@ -110,19 +119,29 @@ class XumlClassDiagram:
 
     def draw_classes(self) -> Dict[str, SingleCellNode]:
         """Draw all of the classes on the class diagram"""
+
         nodes = {}
-        np = self.layout.node_placement
+        np = self.layout.node_placement # Layout data for all classes
+
         for c in self.subsys.classes:
+
+            # Get the class name from the model
             cname = c['name']
             self.logger.info(f'Processing class: {cname}')
+
+            # Get the layout data for this class
             nlayout = np[cname]
+
+            # Layout data for all placements
+            # By default the class name is all on one line, but it may be wrapped across multiple
             nlayout['wrap'] = nlayout.get('wrap', 1)
+            # There is an optional keyletter (class name abbreviation) displayed as {keyletter}
+            # after the class name
             keyletter = c.get('keyletter')
             keyletter_display = f' {{{keyletter}}}' if keyletter else ''
+            # Class name and optional keyletter are in the same wrapped text block
             name_block = TextBlock(cname+keyletter_display, nlayout['wrap'])
-            h = HorizAlign[nlayout.get('halign', 'CENTER')]
-            v = VertAlign[nlayout.get('valign', 'CENTER')]
-            # If this is an imported class, append the import reference to the attribute list
+            # Class might be imported. If so add a reference to subsystem or TBD in attr compartment
             import_subsys_name = c.get('import')
             if not import_subsys_name:
                 internal_ref = []
@@ -130,36 +149,54 @@ class XumlClassDiagram:
                 internal_ref = [' ', f'{import_subsys_name.removesuffix(" TBD")} subsystem', '(not yet modeled)']
             else:
                 internal_ref = [' ', f'(See {import_subsys_name} subsystem)']
-            row_span, col_span = nlayout['node_loc']
-            # If methods were supplied, include them in content
-            # text content includes text for all compartments other than the title compartment
+            # Now assemble all the text content for each class compartment
+            # One list item per compartment in descending vertical order of display
+            # (class name, attributes and optional methods)
             text_content = [name_block.text, c['attributes'] + internal_ref ]
             if c.get('methods'):
                 text_content.append(c['methods'])
-            if len(row_span) == 1 and len(col_span) == 1:
-                nodes[cname] = SingleCellNode(
-                    node_type_name='class' if not import_subsys_name else 'imported class',
-                    content=text_content,
-                    grid=self.flatland_canvas.Diagram.Grid,
-                    row=row_span[0], column=col_span[0],
-                    local_alignment=Alignment(vertical=v, horizontal=h)
-                )
-            else:
-                # Span might be only 1 column or row
-                low_row = row_span[0]
-                high_row = low_row if len(row_span) == 1 else row_span[1]
-                left_col = col_span[0]
-                right_col = left_col if len(col_span) == 1 else col_span[1]
-                nodes[cname] = SpanningNode(
-                    node_type_name='class' if not import_subsys_name else 'imported class',
-                    content=text_content,
-                    grid=self.flatland_canvas.Diagram.Grid,
-                    low_row=low_row, high_row=high_row,
-                    left_column=left_col, right_column=right_col,
-                    local_alignment=Alignment(vertical=v, horizontal=h)
-                )
+
+            # The same class may be placed more than once so that the connectors
+            # have less bends and crossovers. This is usually, but not limited to,
+            # the placement of imported classes. Since we generate the diagram
+            # from a single model, there is no harm in duplicating the same class on a
+            # diagram.
+
+            for i, p in enumerate(nlayout['placements']):
+                h = HorizAlign[p.get('halign', 'CENTER')]
+                v = VertAlign[p.get('valign', 'CENTER')]
+                # If this is an imported class, append the import reference to the attribute list
+                row_span, col_span = p['node_loc']
+                # If methods were supplied, include them in content
+                # text content includes text for all compartments other than the title compartment
+                # When drawing connectors, we want to attach to a specific node placement
+                # In most cases, this will just be the one and only indicated by the node name
+                # But if a node is duplicated, i will not be 0 and we add a suffix to the node
+                # name for the additional placement
+                node_name = cname if i == 0 else f'{cname}_{i+1}'
+                if len(row_span) == 1 and len(col_span) == 1:
+                    nodes[node_name] = SingleCellNode(
+                        node_type_name='class' if not import_subsys_name else 'imported class',
+                        content=text_content,
+                        grid=self.flatland_canvas.Diagram.Grid,
+                        row=row_span[0], column=col_span[0],
+                        local_alignment=Alignment(vertical=v, horizontal=h)
+                    )
+                else:
+                    # Span might be only 1 column or row
+                    low_row = row_span[0]
+                    high_row = low_row if len(row_span) == 1 else row_span[1]
+                    left_col = col_span[0]
+                    right_col = left_col if len(col_span) == 1 else col_span[1]
+                    nodes[node_name] = SpanningNode(
+                        node_type_name='class' if not import_subsys_name else 'imported class',
+                        content=text_content,
+                        grid=self.flatland_canvas.Diagram.Grid,
+                        low_row=low_row, high_row=high_row,
+                        left_column=left_col, right_column=right_col,
+                        local_alignment=Alignment(vertical=v, horizontal=h)
+                    )
         return nodes
-        # TODO:  Include method section in content
         # TODO:  Add support for axis offset on stem names
 
     def draw_association(self, rnum, association, binary_layout):
@@ -168,10 +205,13 @@ class XumlClassDiagram:
         tstem = binary_layout['tstem']
         pstem = binary_layout['pstem']
         astem = binary_layout.get('tertiary_node', None)
+
         t_side = association['t_side']
-        if tstem['name'] != t_side['cname']:
+        if tstem['node_ref'][0] != t_side['cname']:
             # The user put the tstems in the wrong order in the layout file
             # Swap them
+            # The node_ref is a list and the first element refers to the model class name
+            # (the 2nd element indicates duplicate placement, if any, and is not relevant for the comparison above)
             tstem, pstem = pstem, tstem
             self.logger.warning(f"Stems order in layout file does not match model, swapping stem order for connector {rnum}")
 
@@ -179,20 +219,26 @@ class XumlClassDiagram:
             text=TextBlock(t_side['phrase'], wrap=tstem['wrap']),
             side=tstem['stem_dir'], axis_offset=None, end_offset=None
         )
+        node_ref = make_node_ref(tstem['node_ref'])
         t_stem = New_Stem(stem_type='class mult', semantic=t_side['mult'] + ' mult',
-                          node=self.nodes[t_side['cname']], face=tstem['face'],
+                          node=self.nodes[node_ref], face=tstem['face'],
                           anchor=tstem.get('anchor', None), stem_name=t_phrase)
+
+        # Same as for the t_side, but with p instead
         p_side = association['p_side']
         p_phrase = StemName(
             text=TextBlock(p_side['phrase'], wrap=pstem['wrap']),
             side=pstem['stem_dir'], axis_offset=None, end_offset=None
         )
+        node_ref = make_node_ref(pstem['node_ref'])
         p_stem = New_Stem(stem_type='class mult', semantic=p_side['mult'] + ' mult',
-                          node=self.nodes[p_side['cname']], face=pstem['face'],
+                          node=self.nodes[node_ref], face=pstem['face'],
                           anchor=pstem.get('anchor', None), stem_name=p_phrase)
+        # There is an optional stem for an association class
         if astem:
+            node_ref = make_node_ref(astem['node_ref'])
             a_stem = New_Stem(stem_type='associative mult', semantic=association['assoc_mult'] + ' mult',
-                              node=self.nodes[association['assoc_cname']], face=astem['face'], anchor=astem.get('anchor', None),
+                              node=self.nodes[node_ref], face=astem['face'], anchor=astem.get('anchor', None),
                               stem_name=None)
         else:
             a_stem = None
@@ -275,9 +321,9 @@ class XumlClassDiagram:
         """
         One of the rare times it is a good idea to draw one – LS
         """
-        super_name = generalization['superclass']
-        trunk_node = self.nodes[super_name]
         trunk_layout = tree_layout['trunk_face']
+        node_ref = make_node_ref(trunk_layout['node_ref'])
+        trunk_node = self.nodes[node_ref]
 
         # Process trunk branch
         trunk_stem = New_Stem(stem_type='superclass', semantic='superclass', node=trunk_node,
